using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace DeleteElements {

   public partial class frmDeleteElements : Form {

      public frmDeleteElements() {
         InitializeComponent();
         this.Location = Screen.PrimaryScreen.WorkingArea.Location;
      }

      private void btClear_Click(object sender, EventArgs e) {
         lbOutput.Text = "";
      }

      private List<int> GetNumbs() {
         if(txtInput.Text == "") return new List<int>();
         return new List<int>(Array.ConvertAll<string, int>(txtInput.Text.Split(), int.Parse));
      }

      private void SetNumbs(List<int>numbs) {
         lbOutput.Text = string.Join(" ", Array.ConvertAll<int, string>(numbs.ToArray(), Convert.ToString));
      } 

      private void btForEach_Click(object sender, EventArgs e) {
         var numbs = GetNumbs();
         //erzeugt Laufzeit-Fehler
         foreach(int n in numbs) if(n % 3 == 0) numbs.Remove(n);
         SetNumbs(numbs);
      }

      private void btForward_Click(object sender, EventArgs e) {
         var numbs = GetNumbs();
         //tckischer semantischer Fehler: bei aufeinander folgenden Treffern wird der 2. nicht gefunden
         for(int i = 0; i < numbs.Count; i++) if(numbs[i] % 3 == 0) numbs.RemoveAt(i);
         SetNumbs(numbs);
      }

      private void btBackwards_Click(object sender, EventArgs e) {
         var numbs = GetNumbs();
         //simple Lsung - aber jede Lschung lt intern alle Nachfolger vorrcken
         for(int i = numbs.Count; i-- > 0; ) if(numbs[i] % 3 == 0) numbs.RemoveAt(i);
         SetNumbs(numbs);
      }

      private void btReplaceList_Click(object sender, EventArgs e) {
         var numbs = GetNumbs();
         //das verrcken vermeiden, indem eine neue Liste aufgebaut wird. Untauglich, wenn anderweitig
         // weiter Referenzen auf die Original-Liste bestehen bleiben
         var list2 = new List<int>();
         //die Liste wird umkopiert, Treffer werden dabei ausgelassen
         foreach(int n in numbs) if(n % 3 != 0) list2.Add(n);
         SetNumbs(list2);
      }

      private void btOptimized_Click(object sender, EventArgs e) {
         var numbs = GetNumbs();
         for(int i = 0; i < numbs.Count; i++) {
            if(numbs[i] % 3 == 0) {
               /*Die Treffer werden nicht entfernt, sondern ab dem ersten Treffer werden die folgenden 
                * Elemente nach vorn kopiert - Treffer werden dadurch berschrieben */
               int i2 = i;
               for(i++; i < numbs.Count; i++) {
                  if(numbs[i] % 3 != 0) numbs[i2++] = numbs[i];
               }
               //erst ganz zuletzt werden die berzhligen Stellen vom Ende her entfernt
               numbs.RemoveRange(i2, i - i2);
            }
         }
         SetNumbs(numbs);
      }

      private void btRemoveAll_Click(object sender, EventArgs e) {
         // Knigsweg: natrlich die built-in-Methode verwenden! (gibts blos nicht in allen Auflistungen)
         var numbs = GetNumbs();
         //numbs.RemoveAll(delegate(int n) { return n % 3 == 0; });
         numbs.RemoveAll(n => n % 3 == 0);
         SetNumbs(numbs);
      }

      private void btRemoveLabelBackLoop_Click(object sender, EventArgs e) {
         var ctrls = grpMultiControls.Controls;
         grpMultiControls.SuspendLayout();
         //Aus ControlCollection lschen per Rckwrts-Schleife 
         for(int i = ctrls.Count; i-- > 0; ) if(ctrls[i] is Label) ctrls.RemoveAt(i);
         grpMultiControls.ResumeLayout();
      }

      private void btRemoveLabelsByLinq_Click(object sender, EventArgs e) {
         grpMultiControls.SuspendLayout();
         //Aus ControlCollection lschen mit Linq-Methoden 
         foreach(Control c in grpMultiControls.Controls.OfType<Label>().ToArray()) c.Dispose();
         //grpMultiControls.Controls.OfType<Label>().ToList().ForEach(c => c.Dispose());
         grpMultiControls.ResumeLayout();
      }

   }
}

